package cn.iocoder.yudao.module.shipment.listener;


import cn.iocoder.yudao.framework.apollo.core.event.BoxCheckOrderDestWarehouseEvent;
import cn.iocoder.yudao.framework.apollo.core.event.BoxCheckOrderSchedulingEvent;
import cn.iocoder.yudao.framework.apollo.core.event.BoxCheckOrdersSchedulingEvent;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseLineMapper;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.WAREHOUSE_IN_BOX_EXCEPTION;

/**
 * 提供订单是否已经排单校验的接口，如果已经排单，则不能货物补充（会导致已装体积重量的不正确），需要先从预装单中移除才能继续补充，
 *
 * @author zhengYi
 */
@Component("BoxCheckOrderSchedulingListener")
@AllArgsConstructor
@Slf4j
public class BoxCheckOrderSchedulingListener {

    private final BoxService boxService;

    private final OrderService orderService;

    private final WarehouseLineMapper warehouseLineMapper;

    /**
     * 提供订单是否已经排单校验的接口，如果已经排单，则不能货物补充（会导致已装体积重量的不正确），需要先从预装单中移除才能继续补充，
     *
     * @param event 订单号
     */
    @EventListener(BoxCheckOrderSchedulingEvent.class)
    public void checkOrderScheduling(BoxCheckOrderSchedulingEvent event) {
        Long orderId = event.getOrderId();
        log.info("提供订单是否已经排单校验的接口 orderId={}", orderId);
        BoxDO boxDO = boxService.checkOrderScheduling(orderId);
        if (boxDO != null) {
            throw exception(WAREHOUSE_IN_BOX_EXCEPTION);
        }

    }

    /**
     * 提供多订单订单是否已经排单校验的接口
     * 返回已排单的订单id
     *
     * @param event 订单号
     */
    @EventListener(BoxCheckOrdersSchedulingEvent.class)
    public void checkOrdersScheduling(BoxCheckOrdersSchedulingEvent event) {
        List<Long> orderIds = event.getOrderIds();
        log.info("提供订单是否已经排单校验的接口 orderIds={}", JSONObject.toJSONString(orderIds));
        List<Long> schedulingOrderIds = boxService.checkOrdersScheduling(orderIds);
        event.setSchedulingOrderIds(schedulingOrderIds);

    }



    /**
     * 提供订单是否已经排单校验的接口，如果已经排单，则不能货物补充（会导致已装体积重量的不正确），需要先从预装单中移除才能继续补充，
     *
     * @param event 订单号
     */
    @EventListener(BoxCheckOrderDestWarehouseEvent.class)
    public void checkOrderDestWarehouse(BoxCheckOrderDestWarehouseEvent event) {
        Long orderId = event.getOrderId();
        log.info("提供订单目的仓库是否跟卸柜仓库目的仓一致 orderId={}", orderId);
        BoxDO boxDO = boxService.checkOrderScheduling(orderId);
        if (boxDO != null) {
            OrderDO orderDO = orderService.getById(orderId);
            if (orderDO != null) {
                WarehouseLineDO warehouseLineDO = warehouseLineMapper.selectOne(new LambdaQueryWrapper<WarehouseLineDO>().eq(WarehouseLineDO::getId, orderDO.getLineId()).eq(WarehouseLineDO::getTransportType, orderDO.getTransportId()).eq(BaseDO::getDeleted, false));

                event.setBoxDestWarehouseIdReturn(boxDO.getDestWarehouseId());
                event.setOrderDestWarehouseIdReturn(warehouseLineDO.getDestWarehouseId());

                event.setIsSameDestWarehouseReturn(Objects.equals(warehouseLineDO.getDestWarehouseId(), boxDO.getDestWarehouseId()));
            }
        }

    }
}
