package cn.iocoder.yudao.module.shipment.service.boxAirNumberRecord;

import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.QueryChannelInfoEvent;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxAirNumberRecord.BoxAirNumberRecordDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.boxAirNumberRecord.BoxAirNumberRecordMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.CITY_NOT_SET_LABEL_CODE;

/**
 * 空运出货单编号记录 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxAirNumberRecordServiceImpl extends AbstractService<BoxAirNumberRecordMapper, BoxAirNumberRecordDO> implements BoxAirNumberRecordService {

    @Resource
    private BoxAirNumberRecordMapper boxAirNumberRecordMapper;
    @Resource
    private RegionService regionService;

    @Resource
    private ApplicationContext applicationContext;

    @Override
    public BoxAirNumberRecordDO getBoxAirNumberRecord(Long id) {
        return boxAirNumberRecordMapper.selectById(id);
    }

    @Override
    public List<BoxAirNumberRecordDO> getBoxAirNumberRecordList(Collection<Long> ids) {
        return boxAirNumberRecordMapper.selectBatchIds(ids);
    }


    @Override
    public void updateBoxAirNumberRecordUsedStatus(String number) {
        this.update(new LambdaUpdateWrapper<BoxAirNumberRecordDO>()
                .set(BoxAirNumberRecordDO::getUsed, false)
                .eq(BoxAirNumberRecordDO::getNumber, number));
    }

    @Override
    public BoxAirNumberRecordDO getBoxAirNumberRecord(Integer transportType, Long destId, Long channelId) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        // 获取目的地的国家简码和渠道的简码
        RegionDO destRegion = regionService.getRegion(destId);
        if (Objects.isNull(destRegion)) {
            throw exception(OBJECTIVE_NOT_EXISTS);
        }
        if (StringUtils.isBlank(destRegion.getLabelCode())) {
            throw exception(CITY_NOT_SET_LABEL_CODE, I18nMessage.getLang() == 0 ? destRegion.getTitleZh() : destRegion.getTitleEn());
        }
        String[] labelCodes = destRegion.getLabelCode().split(StrUtil.DASHED);
        // 空运订单编号国家简称在目的地配置国家简称的第三位
        if (transportType == 3 && labelCodes.length < 4) {
            throw exception(CITY_NOT_SET_AIR_SELF_LABEL_CODE, I18nMessage.getLang() == 0 ? destRegion.getTitleZh() : destRegion.getTitleEn());
        }
        String destCountryCode = labelCodes[3];

        if (Objects.isNull(channelId) || channelId == 0) {
            throw exception(ORDER_LINE_CHANNEL_NOT_NULL);
        }
        QueryChannelInfoEvent event = new QueryChannelInfoEvent();
        event.setChannelId(channelId);
        applicationContext.publishEvent(event);
        if (StringUtils.isBlank(event.getCode())) {
            throw exception(CHANNEL_CODE_NOT_SET, I18nMessage.getLang() == 0 ? event.getNameZh() : event.getNameEn());
        }
        String channelCode = event.getCode();
        BoxAirNumberRecordDO boxAirNumberRecordDO = boxAirNumberRecordMapper.selectOne(new LambdaQueryWrapper<BoxAirNumberRecordDO>()
                .eq(BoxAirNumberRecordDO::getTransportType, transportType)
                .eq(BoxAirNumberRecordDO::getDestCountryCode, destCountryCode)
                .eq(BoxAirNumberRecordDO::getUsed, 0)
                .orderByAsc(BoxAirNumberRecordDO::getId).last("limit 1"));
        if (Objects.nonNull(boxAirNumberRecordDO)) {
            // 修改自编号逻辑
            if (!StringUtils.equals(channelCode, boxAirNumberRecordDO.getChannelCode())) {
                String newNumber = CodeUtils.chargeChannelAirSelfNumbers(destCountryCode, channelCode, boxAirNumberRecordDO.getChannelCode(), boxAirNumberRecordDO.getNumber());
                boxAirNumberRecordDO.setDestId(destId);
                boxAirNumberRecordDO.setDestCountryCode(destCountryCode);
                boxAirNumberRecordDO.setNumber(newNumber);
                boxAirNumberRecordDO.setChannelCode(channelCode);
                boxAirNumberRecordDO.setChannelId(channelId);
            }
            // 如果有未使用的编号记录，则直接更新为使用状态后返回
            boxAirNumberRecordDO.setUsed(Boolean.TRUE);
            boxAirNumberRecordDO.setUpdater(String.valueOf(userId));
            boxAirNumberRecordDO.setUpdateTime(new Date());
            boxAirNumberRecordMapper.updateById(boxAirNumberRecordDO);
            return boxAirNumberRecordDO;
        }
        // 当没有未使用的编号记录时，需要生成最新的编号记录
        return generateNewBoxAirNumberRecordDO(transportType, destId, channelId, destCountryCode, channelCode, userId);
    }


    @Override
    public BoxAirNumberRecordDO chargeChannelBoxAirNumberRecord(String number, Integer transportType, Long destId, Long channelId) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        // 获取目的地的国家简码和渠道的简码
        RegionDO destRegion = regionService.getRegion(destId);
        if (Objects.isNull(destRegion)) {
            throw exception(OBJECTIVE_NOT_EXISTS);
        }
        if (StringUtils.isBlank(destRegion.getLabelCode())) {
            throw exception(CITY_NOT_SET_LABEL_CODE, I18nMessage.getLang() == 0 ? destRegion.getTitleZh() : destRegion.getTitleEn());
        }
        String[] labelCodes = destRegion.getLabelCode().split(StrUtil.DASHED);
        // 空运订单编号国家简称在目的地配置国家简称的第三位
        if (transportType == 3 && labelCodes.length < 4) {
            throw exception(CITY_NOT_SET_AIR_SELF_LABEL_CODE, I18nMessage.getLang() == 0 ? destRegion.getTitleZh() : destRegion.getTitleEn());
        }
        String destCountryCode = labelCodes[3];

        if (Objects.isNull(channelId) || channelId == 0) {
            throw exception(ORDER_LINE_CHANNEL_NOT_NULL);
        }
        QueryChannelInfoEvent event = new QueryChannelInfoEvent();
        event.setChannelId(channelId);
        applicationContext.publishEvent(event);
        if (StringUtils.isBlank(event.getCode())) {
            throw exception(CHANNEL_CODE_NOT_SET, I18nMessage.getLang() == 0 ? event.getNameZh() : event.getNameEn());
        }
        String channelCode = event.getCode();
        // 获取当前需要修改的自编号记录
        BoxAirNumberRecordDO boxAirNumberRecordDO = boxAirNumberRecordMapper.selectOne(new LambdaQueryWrapper<BoxAirNumberRecordDO>()
                .eq(BoxAirNumberRecordDO::getNumber, number)
                .eq(BoxAirNumberRecordDO::getTransportType, transportType)
                .eq(BoxAirNumberRecordDO::getDestCountryCode, destCountryCode)
                .orderByAsc(BoxAirNumberRecordDO::getId).last("limit 1"));
        if (Objects.nonNull(boxAirNumberRecordDO)) {
            // 修改自编号逻辑
            String newNumber = CodeUtils.chargeChannelAirSelfNumbers(destCountryCode, channelCode, boxAirNumberRecordDO.getChannelCode(), boxAirNumberRecordDO.getNumber());
            // 如果有未使用的编号记录，则直接更新为使用状态后返回
            boxAirNumberRecordDO.setDestId(destId);
            boxAirNumberRecordDO.setDestCountryCode(destCountryCode);
            boxAirNumberRecordDO.setNumber(newNumber);
            boxAirNumberRecordDO.setChannelCode(channelCode);
            boxAirNumberRecordDO.setChannelId(channelId);
            boxAirNumberRecordDO.setUsed(Boolean.TRUE);
            boxAirNumberRecordDO.setUpdater(String.valueOf(userId));
            boxAirNumberRecordDO.setUpdateTime(new Date());
            boxAirNumberRecordMapper.updateById(boxAirNumberRecordDO);
            return boxAirNumberRecordDO;
        }

        // 当没有未使用的编号记录时，需要生成最新的编号记录
        return generateNewBoxAirNumberRecordDO(transportType, destId, channelId, destCountryCode, channelCode, userId);
    }

    @NotNull
    private BoxAirNumberRecordDO generateNewBoxAirNumberRecordDO(Integer transportType, Long destId, Long channelId, String destCountryCode, String channelCode, Long userId) {
        BoxAirNumberRecordDO boxAirNumberRecordDO;
        // 先获取最大的自编号
        BoxAirNumberRecordDO maxBoxAirNumberRecordDO = boxAirNumberRecordMapper.selectOne(new LambdaQueryWrapper<BoxAirNumberRecordDO>()
                .eq(BoxAirNumberRecordDO::getTransportType, transportType)
                .eq(BoxAirNumberRecordDO::getDestCountryCode, destCountryCode)
                .orderByDesc(BoxAirNumberRecordDO::getId).last("limit 1"));
        // 根据最大的自编号生成最新的自编号
        String preChannelCode = "";
        String maxOddNumbers = "";
        if (Objects.nonNull(maxBoxAirNumberRecordDO)){
            preChannelCode = maxBoxAirNumberRecordDO.getChannelCode();
            maxOddNumbers = maxBoxAirNumberRecordDO.getNumber();
        }
        // 生成自编号逻辑
        String newNumber = CodeUtils.getOddAirSelfNumbers(destCountryCode, channelCode, preChannelCode, maxOddNumbers);
        // 添加最新的自编号记录
        boxAirNumberRecordDO = new BoxAirNumberRecordDO();
        boxAirNumberRecordDO.setNumber(newNumber);
        boxAirNumberRecordDO.setChannelCode(channelCode);
        boxAirNumberRecordDO.setChannelId(channelId);
        boxAirNumberRecordDO.setTransportType(transportType);
        boxAirNumberRecordDO.setUsed(Boolean.TRUE);
        boxAirNumberRecordDO.setDestId(destId);
        boxAirNumberRecordDO.setDestCountryCode(destCountryCode);
        boxAirNumberRecordDO.setCreator(String.valueOf(userId));
        boxAirNumberRecordDO.setCreateTime(new Date());
        boxAirNumberRecordDO.setUpdater(String.valueOf(userId));
        boxAirNumberRecordDO.setUpdateTime(new Date());
        boxAirNumberRecordMapper.insert(boxAirNumberRecordDO);
        return boxAirNumberRecordDO;
    }
}
